﻿/*
n80: 8.31 g/cm3
*/

function mm2in(n) { return n / 25.4; }
function in2mm(n) { return n * 25.4; }
function cm2in(n) { return n / 2.54; }
function in2cm(n) { return n * 2.54; }
function g2oz(n) { return n / 28.3495231; }
function oz2g(n) { return n * 28.3495231; }
function g2gr(n) { return n / 0.06479891; }
function gr2g(n) { return n * 0.06479891; }
var in2 = 25.4*25.4;
var in3 = 25.4*in2;

function awg2in(awg) {
	return 0.005 * Math.pow(92,(36-awg)/39);
}
function awg2mm(awg) {
	return in2mm(awg2in(awg));
}

function defaultCoil() {
	return {
		//ver: '1.0'
		showSi : true
		,advanced : false
		,mat : 'ka1'
		,profile : 'round'
		,strands: 1
		,twistpitch: 0
		,awg : 30
		,setup : 's'
		,targetr : 1.8
		,rstv : 0.028472674975859414
		,wirelength : 63.21850692027117
		,dia : 0.2546390029766585
		,width: 0.5
		,thickness: 0.1
		,innerdia : 1.3
		,leglength : 5
		,wrapspace : 0.01
		,useNaDia: false
		,hfnw: 10
	};
}

var coil = defaultCoil();
coil.converter = {};
coil.defaultCoil = defaultCoil();
var aux = {};

var materials = {
	/*
	// heat capacity i J/Kg-K (???) - noen heat capacities for grunnstoffer:
	fe: 0.449
	cr: 0.449
	al: 0.897
	ni: 0.444
	*/
	ka1 : {
		rstv: 1.45 // (Ω mm²/m)
		,hc: 0.46 // typisk kanthal, og oppgitt for ka1
		,dens: 7.1 // gram pr ccm
	}
	,ka : {
		rstv: 1.39
		,hc: 0.4625 // guesstimate
		,dens: 7.15
	}
	,kd : {
		rstv: 1.35
		,hc: 0.465 // guesstimate
		,dens: 7.25
	}
	,n20 : { // 20% krom, 80% nikkel
		rstv: 0.95
		,hc: 0.447 // alle grunstoffene i nikrom er mellom 0.444 og 0.449
		,dens: 8.7 // guesstimate
	}
	,n40 : {
		rstv: 1.04
		,hc: 0.447
		,dens: 8.6 // interpolert
	}
	,n60 : {
		rstv: 1.11
		,hc: 0.447
		,dens: 8.5 // interpolert
	}
	,n70 : {
		rstv: 1.18
		,hc: 0.447
		,dens: 8.4 // interpolert
	}
	,n80 : { // 80% krom, 20% nikkel
		rstv: 1.09
		,hc: 0.447
		,dens: 8.31
	}
	,ss316l : {
		rstv: 0.75
		,hc: 0.5
		,dens: 8
	}
	,ss317l : {
		rstv: 0.81  // 0.87? 0.79?
		,hc: 0.5
		,dens: 7.9
	}
	,ti1 : {
		rstv: 0.47
		,hc: 0.523
		,dens: 4.43
	}
	,ti2 : { // R50400
		rstv: 0.56
		,hc: 0.54 // guesstimate
		,dens: 4.51
	}
	,tie : { // "Titanium ready" from e-SmokeGuru
		/*
		- 0.18mm wire at 33.4 Ω/m gives 0.85 Ω.mm²/m,
		- 0.20mm wire at 22.5 Ω/m gives 0.71 Ω.mm²/m,
		- 0.25mm wire at 19.5 Ω/m gives 0.96 Ω.mm²/m,
		- 0.32mm wire at 13.3 Ω/m gives 1.07 Ω.mm²/m,
		- 0.40mm wire at 6.8 Ω/m gives 0.85 Ω.mm²/m.				
		*/
		rstv: 0.85
		,hc: 0.56 // guesstimate
		,dens: 4.75 // guesstimate
	}
	,nio : {
		rstv: (1.01247892074 * 1.45) // niobium har iflg. StealthVape en resistivitet på 1.01247892074 ganger Kanthal A1 med samme tykkelse.
		,hc: 0.45 // wild guess
		,dens: 7.5 // wild guess
	}
	,ni200 : {
		// rstv er
		// 0.096 ved 20 C
		// 0.13 ved 100 C
		// 0.185 ved 200 C
		rstv: 0.096
		,hc: 0.456
		,dens: 8.89
	}
	,nife30 : {
		rstv: 0.33 // http://www.abcpol.pl/en/our-offer/resistance-alloys
		,hc: 0.45 // guesstimate
		,dens: 8.2 // guesstimate
	}
	,c : {	// Custom resistivity - fallback-verdier for heat capacity og density
		hc: 0.5
		,dens: 8.31
	}
};

function findXArea(spole) {
	if (spole.profile==='ribbon') {
		return spole.width * spole.thickness;
	}
	//} else {
	return Math.PI * (spole.dia/2) * (spole.dia/2) * spole.strands;
	//}
}

function ds(nr, a, b) { // bruker coil-objektet
	if (coil.showSi) {
		if (a === undefined) { a = 3;}
		return round(nr,a);
	}
	if (b === undefined) { b = 5;}
	return round(mm2in(nr),b);
}
function writeRstv(spole) { // motsatt konvertering siden lengden er i nevneren
	if (spole.showSi) {
		$('#cd-rstv').val(round(spole.rstv,5)).parents('.form-group');
	} else {
		$('#cd-rstv').val(round(in2mm(spole.rstv),4)).parents('.form-group');
	}
	return;
}

function findResistivity(spole) {
	if (spole.mat === 'c') { return; }
	var tmpRes = materials[spole.mat].rstv / 1000;
	var rstv = tmpRes / findXArea(spole);
	coil.rstv = rstv;
	if (coil === spole) {
		writeRstv(spole);
	}
	return;
}
function getRfactor(spole) {
	if (spole.setup==='s'||spole.setup==='qsp') { return 1; }
	if (spole.setup==='dp') { return 0.5; } 
	if (spole.setup==='tp') { return 1/3; } 
	if (spole.setup==='qp') { return 0.25; } 
	if (spole.setup==='5p') { return 0.2; } 
	if (spole.setup==='6p') { return 1/6; } 
	if (spole.setup==='7p') { return 1/7; } 
	if (spole.setup==='8p') { return 1/8; } 
	if (spole.setup==='ds') { return 2; } 
	if (spole.setup==='ts') { return 3; } 
	if (spole.setup==='qs') { return 4; } 
	console.log('Warning: getRfactor(spole) failed, unknown setup ['+spole.setup+'].');
	return;
}

function findTwist(spole) { // kan også modifisere spole.rstv
	if (spole.twistpitch === 0) {
		spole.twistfactor = 1;
	} else if (spole.strands > 1) {
		// beregne heliks
		spole.twistlead = spole.twistpitch * spole.strands; // antall mm på langs per rotasjon, per tråd
		var tmpNeutralAxisDia = spole.dia;
		var tmpCircum = tmpNeutralAxisDia * Math.PI;
		spole.twisthypotenus = Math.sqrt((tmpCircum*tmpCircum)+(spole.twistlead*spole.twistlead));
		// modifisere resistivitet
		spole.twistfactor = (spole.twisthypotenus/spole.twistlead);
		spole.rstv *= spole.twistfactor;
		if (spole === coil) {
			writeRstv(spole);
			//$('#cd-rstv').val(ds(spole.rstv,4,4)).parents('.form-group').removeClass('has-warning');
		}
	} else {
		spole.twistfactor = 1;
	}
	return;
}

function klon(o) {
	var k = {};
	var x;
	for (x in o) {
		if (o.hasOwnProperty(x)) {
			//console.log([x]);
			k[x] = o[x];
		}
	}
	return k;
}

function wireLength(spole) {
	spole.wirelength = (spole.targetr/spole.rstv);
	spole.wirelength /= getRfactor(spole); // single coil, dual, triple, etc.
	if (coil === spole) {
		$('#cd-len').val(ds(spole.wirelength,2,3));
	}
	return;
}

function exportSettings() {
	var dc = defaultCoil();
	var exp = {};
	function expIfNotDef(foo) {
		if (coil[foo] !== dc[foo]) {
			if (foo==='leglength') {
				exp.ll = coil.leglength;
			} else if (foo==='innerdia') {
				exp.id = coil.innerdia;
			} else if (foo==='targetr') {
				exp.r = coil.targetr;
			} else if (foo==='setup') {
				exp.s = coil.setup;
			} else if (foo==='profile') {
				exp.p = coil.profile;
			} else if (foo==='twistpitch') {
				exp.tp = coil.twistpitch;
			} else if (foo==='strands') {
				exp.str = coil.strands;
			} else if (foo==='wirelength') {
				exp.wl = coil.wirelength;
			} else if (foo==='wrapspace') {
				exp.ws = coil.wrapspace;
			} else if (foo==='hfnw') {
				exp.hfnw = coil.hfnw;
			} else {
				exp[foo] = coil[foo];
			}
		}
		return;
	}
	// eksporter boolske grunnverdier hvis nødvendig
	if (!coil.showSi) { exp.si = false; }
	if (coil.advanced) { exp.a = true; }
	if (coil.useNaDia) { exp.useNaDia = true; }
	
	// eksporter grunnverdier hvis forskjellig fra default
	if (coil.mat==='c') {
		exp.rstv = coil.rstv;
	} else {
		expIfNotDef('mat');
	}
	expIfNotDef('profile');
	expIfNotDef('twistpitch');
	expIfNotDef('setup');
	expIfNotDef('targetr');
	expIfNotDef('hfnw');
	
	if (coil.profile==='ribbon') {
		expIfNotDef('width');
		expIfNotDef('thickness');
	} else {
		expIfNotDef('strands');
		if (coil.awg === '') {
			expIfNotDef('dia');
		} else {
			expIfNotDef('awg');
		}
	}

	// eksporter wirelength hvis forskjellig fra kalkulert
	var tmpCoil = klon(coil);

	findResistivity(tmpCoil);
	wireLength(tmpCoil);
	if (coil.wirelength !== tmpCoil.wirelength) {
		exp.wl = coil.wirelength;
	}
	expIfNotDef('innerdia');
	expIfNotDef('leglength');
	expIfNotDef('wrapspace');
	var url = '';
	var x;
	dc = defaultCoil();
	for (x in exp) {
		if (exp[x]!=='' && exp[x]!==dc[x]) {
			url = url + '&' + x + '=' + exp[x];
		}
	}
	updateQs(url);
	return;
}

function wraps() {
	$('.cd-calcicon').show();
	clearTimeout(aux.calculatingtimeout);
	aux.calculatingtimeout = setTimeout(function() {
		$('.cd-calcicon').fadeOut(250);
	},300);
	var outerdia;
	var neutralaxisdia;
	var twistdia = coil.dia;
	if (coil.profile === 'ribbon') {
		outerdia = coil.innerdia+(coil.thickness*2);
		neutralaxisdia = coil.innerdia+coil.thickness;
	} else {
		if (coil.twistpitch !== 0) {
			if (coil.strands===2) {
				twistdia *= 2;
			} else if (coil.strands===3) {
				twistdia /= (2 * Math.sqrt(3)) - 3; // tre tangerende sirkler i en sirkel
			} else if (coil.strands===4) {
				twistdia *= 1 + Math.sqrt(2); // fire tangerende sirkler i en sirkel
			}
		}
		outerdia = coil.innerdia + (twistdia*2);
		neutralaxisdia = coil.innerdia + twistdia;
	}
	var loopcircum;
	if (coil.useNaDia) {
		loopcircum = (neutralaxisdia * Math.PI);
	} else {
		loopcircum = (outerdia * Math.PI);
	}
	var looplength;
	var coillead;
	if (coil.profile === 'ribbon') {
		coillead = coil.wrapspace + coil.width;
	} else if (coil.twistpitch === 0) {
		coillead = (coil.wrapspace + coil.dia) * coil.strands;
	} else {
		coillead = coil.wrapspace + twistdia;
	}
	looplength = Math.sqrt((loopcircum*loopcircum) + (coillead*coillead));
	var coilresistance = coil.targetr / getRfactor(coil);
	var helixangle;
	if (coil.profile === 'ribbon') {
		helixangle = Math.atan2(coil.wrapspace+coil.width, loopcircum) * 180 / Math.PI;	
	} else if (coil.twistpitch === 0) {
		helixangle = Math.atan2((coil.wrapspace + coil.dia) * coil.strands, loopcircum) * 180 / Math.PI;	
	} else {
		helixangle = Math.atan2(coil.wrapspace+twistdia, loopcircum) * 180 / Math.PI;	
	}
	var coillength = coil.wirelength - coil.leglength;
	var numberOfWraps = coillength/looplength;
	var roundedNumberOfWraps = Math.round(numberOfWraps);
	var coilwidth;
	if (coil.profile === 'ribbon') {
		coilwidth = ((numberOfWraps-1) * (coil.wrapspace+coil.width)) + coil.width;
	} else if (coil.twistpitch === 0) {
		coilwidth = ((numberOfWraps-1) * ((coil.wrapspace + coil.dia) * coil.strands)) + (coil.dia * coil.strands);
	} else {
		coilwidth = ((numberOfWraps-1) * (coil.wrapspace+twistdia)) + twistdia;
	}
	coil.legloss = coil.leglength/coil.wirelength;
	var legRes = coil.legloss * coilresistance; // R i beina
	var coilRes = (1-coil.legloss) * coilresistance; // R i spolen uten bein
	var unitNameLen;
	var udl;
	if (coil.showSi) {
		unitNameLen = 'mm';
		udl = 1;
	} else {
		unitNameLen = 'in';
		udl = 3;
	}
	$('.summary-unmlen').html(unitNameLen);
	$('#summary-wirelength .summary-res').html(ds(coil.wirelength,udl,udl));
	$('#summary-numberOfWraps .summary-res').html(round(numberOfWraps,2));
	var fullWrapRealRes = ((roundedNumberOfWraps/numberOfWraps)*coilRes) + legRes;
	var halfWrapRealRes = (((parseInt(numberOfWraps+1,10)-0.5)/numberOfWraps)*coilRes) + legRes;
	$('#summary-roundedNumberOfWraps .summary-res').html(roundedNumberOfWraps);
	$('#summary-roundedNumberOfWraps .summary-ohm').html(round(fullWrapRealRes));
	$('#summary-halfRoundedNumberOfWraps .summary-res').html(parseInt(numberOfWraps+1,10) + '/' + parseInt(numberOfWraps,10));
	$('#summary-halfRoundedNumberOfWraps .summary-ohm').html(round(halfWrapRealRes));
	if (coil.setup!=='s') {
		$('#summary-coilresistance .summary-res').html(round(coilresistance,2));
	}
	var $tmp;
	$tmp = $('#summary-legloss');
	$tmp.attr('title','').removeClass('warning danger');
	if (coil.legloss > 1) {
		$tmp.attr('title','Legs are too long. Please change your parameters and try again.');
		$tmp.addClass('danger');
	} else if (coil.legloss > 0.3) {
		$tmp.attr('title','You are losing too much heat in the legs. Consider changing your parameters.');
		$tmp.addClass('warning');
	} else if (coil.legloss > 0.2) {
		$tmp.attr('title','You are losing a lot of heat in the legs. Consider changing your parameters.');
		//$tmp.addClass('info');
	}
	$tmp.find('td.summary-res').html(round(coil.legloss*100,1));
	$tmp = $('#summary-coillength .summary-res');
	if (coillength > 0) {
		$tmp.html(ds(coillength,udl,udl));
	} else {
		$tmp.html('<span class="label label-danger" title="Legs are too long. No wire left for coil. Please change your parameters and try again.">' + ds(coillength,udl) + '</span>');
	}
	$('#summary-outerdia .summary-res').html(ds(outerdia,udl+1,udl+1));
	$('#summary-neutralaxisdia .summary-res').html(ds(neutralaxisdia,udl+1,udl+1));
	$('#summary-loopcircum .summary-res').html(ds(loopcircum,udl+1,udl+1));
	$('#summary-helixangle .summary-res').html(round(helixangle,1));
	$('#summary-looplength .summary-res').html(ds(looplength,udl+1,udl+1));
	$('#summary-coilwidth .summary-res').html(ds(coilwidth,udl,udl));
	// HEAT ****
	var coilquota = 1 - coil.legloss; // hvor mye av tråden som er i coilen.
	var wirecircum;
	if (coil.profile === 'ribbon') {
		wirecircum = (2 * coil.width) + (2 * coil.thickness);
	} else {
		wirecircum = coil.dia * Math.PI * coil.strands;
	}
	var surfacearea = coil.wirelength * wirecircum * coil.twistfactor;
	var coilarea = surfacearea * coilquota;
	var watt = $('#hfnw').val();
	if (isNaN(watt)) {
		watt = 1;
		$('#hfnw').val(watt);
	} else {
		watt = readNumber(watt);
	}
	var volt = Math.sqrt(watt*coil.targetr);
	var wireheatfluxp = watt/surfacearea; // watt pr kvadratmeter
	if (coil.setup === 'dp' || coil.setup === 'ds') {
		wireheatfluxp /= 2;
	} else if (coil.setup === 'tp' || coil.setup === 'ts') {
		wireheatfluxp /= 3;
	} else if (coil.setup === 'qp' || coil.setup === 'qs' || coil.setup === 'qsp') {
		wireheatfluxp /= 4;
	} else if (coil.setup === '5p') {
		wireheatfluxp /= 5;
	} else if (coil.setup === '6p') {
		wireheatfluxp /= 6;
	} else if (coil.setup === '7p') {
		wireheatfluxp /= 7;
	} else if (coil.setup === '8p') {
		wireheatfluxp /= 8;
	}
	var wireheatfluxu = volt/surfacearea; // volt pr kvadratmeter
	var xarea = findXArea(coil);
	var wirevol = xarea * coil.wirelength * coil.twistfactor; // coil.strands er allerede regnet inn i coil.xarea
	var density = materials[coil.mat].dens; //getDensity(coil);
	var wiremass = wirevol * density; // milligram
	var specificheatcapacity = materials[coil.mat].hc; // getHeatCapacity(coil);
	var heatcapacity = specificheatcapacity * wiremass;
	if (coil.showSi) {
		$('#summary-surfacearea .summary-res').html(round(surfacearea,2));
		$('#summary-coilarea .summary-res').html(round(coilarea,2));
		$('#summary-wireheatfluxp .summary-res').html(round(wireheatfluxp*1000,0));
		//$('#summary-wireheatfluxu .summary-res').html(round(wireheatfluxu*1000,3));
		$('#summary-xarea .summary-res').html(round(xarea,3));
		$('#summary-wirevolume .summary-res').html(round(wirevol,3));
		$('#summary-density .summary-res').html(round(density,3)); // g/cm3
		$('#summary-wiremass .summary-res').html(round(wiremass,3));
	} else {
		$('#summary-surfacearea .summary-res').html(round(surfacearea/in2,5));
		$('#summary-coilarea .summary-res').html(round(coilarea/in2,5));
		$('#summary-wireheatfluxp .summary-res').html(round(wireheatfluxp*1000*in2,0));
		$('#summary-wireheatfluxu .summary-res').html(round(wireheatfluxu*1000*in2,0));
		$('#summary-xarea .summary-res').html(round(xarea/in2,6));
		$('#summary-wirevolume .summary-res').html(round(wirevol/in3,6));
		$('#summary-density .summary-res').html(round(density*0.578036672,3));
		$('#summary-wiremass .summary-res').html(round(g2gr(wiremass/1000),7));
	}
	$('#summary-heatcapacity .summary-res').html(round(heatcapacity,2));
	
	function fargHeatFlux(heatflux) {
		var $r = $('#heaticon');
		var cArr = ['0098E5', '00ACE4', '00C0E3', '00D4E2', '00E1D9', '00E0C3', '00DFAD', '00DE98', '00DC6E', '00DA44', '33D400', '59D200', '7FD000', 'A4CE00', 'B7CD00', 'C9CC00', 'CBBC00', 'CAA800', 'C99400', 'C88100', 'C76E00', 'C65B00', 'C54800', 'C43500', 'C32300', 'C21100', 'C10000', 'C00012', 'BF0024', 'BF0035'];
		var hf = Math.round(heatflux*80)-4;
		if (hf<1) { hf = 1; }
		if (hf>cArr.length-1) { hf = cArr.length-1; }
		var c = cArr[hf];
		$r.css('background-color','#'+c);
		return;
	}		
	fargHeatFlux(wireheatfluxp);
	
	return;
}
function setupFracInput($inpdiv) {
	var $inp = $inpdiv.find('input').filter(':first');
	var faid = $inpdiv.attr('id');
	var html = '';
	html += '<div class="fractionmenu" id="'+faid+'-fractionmenu">';
		html += '<span class="btn btn-default btn-xs pull-right" id="'+faid+'-fractionbtn"><sup>x</sup>/<sub>y</sub>"</span>';
		html += '<div class="fractionpresets" style="display:none;" id="'+faid+'scd-fractionpresets">';
			html += '<div class="btn-group-justified">';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.25">1/4"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.125">1/8"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.375">3/8"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.0625">1/16"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.1875">3/16"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.3125">5/16"</span>';
			html += '</div>';
			html += '<div class="btn-group-justified">';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.03125">1/32""</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.09375">3/32"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.15625">5/32"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.21875">7/32"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.28125">9/32"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.34375">11/32"</span>';
			html += '</div>';
			html += '<div class="btn-group-justified">';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.015625"><sup>1</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.046875"><sup>3</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.078125"><sup>5</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.109375"><sup>7</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.140625"><sup>9</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.171875"><sup>11</sup>/<sub>64</sub>"</span>';
			html += '</div>';
			html += '<div class="btn-group-justified">';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.203125"><sup>13</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.234375"><sup>15</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.265625"><sup>17</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.296875"><sup>19</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.328125"><sup>21</sup>/<sub>64</sub>"</span>';
				html += '<span class="btn btn-default btn-xs" data-cd-im="0.359375"><sup>23</sup>/<sub>64</sub>"</span>';
			html += '</div>';
		html += '</div>';
	html += '</div>';
	var $helpblock = $inpdiv.find('.help-block').filter(':first');
	if ($helpblock.length===0) {
		$inpdiv.append(html);
	} else {
		$helpblock.before(html);
	}
	var $prebtns = $('#'+faid+'scd-fractionpresets').find('.btn');
	$prebtns.click(function() {
		$prebtns.removeClass('active');
		$(this).addClass('active');
		var p = $(this).attr('data-cd-im');
		if (coil.showSi) {
			p = round(in2mm(p),3);
		} else {
			p = $(this).attr('data-cd-im');
		}
		$inp.val(p).change();
		wraps();
		gad('coil', 'fracinputdiv-' + $inpdiv.attr('id'), $(this).text());
	});
	$('#'+faid+'-fractionbtn').click(function() {
		if (!$(this).hasClass('active')) {
			$('#'+faid+'scd-fractionpresets').slideDown();
		} else {
			$('#'+faid+'scd-fractionpresets').slideUp();
		}
		$(this).toggleClass('active');
	});$('#' + faid).change(function() {
		$prebtns.removeClass('active');
	});
	return;
}

function writeDia() {
	if (coil.dia==='') {
		$('#cd-dia').val('').parents('.form-group').addClass('has-warning');
	} else {
		$('#cd-dia').val(ds(coil.dia,3,5)).parents('.form-group').removeClass('has-warning');
	}
	return;
}

function ccWrite() {
	$('#cd-material').val(coil.mat);
	$('#cd-profile').val(coil.profile);
	//console.log(coil.profile);
	if (coil.profile === 'round') {
		$('.cd-wiredimensions').hide();
		$('.cd-wirediameter').show();
		$('.cd-twistpitch').hide();
		$('.cd-strands').hide();
	} else if (coil.profile === 'roundmulti') {
		$('.cd-wiredimensions').hide();
		$('.cd-wirediameter').show();
		$('#cd-twistpitch').val(ds(coil.twistpitch));
		$('.cd-twistpitch').show();
		$('.cd-twistpitch-output').html(coil.strands);
		$('.cd-strands').show();
	} else if (coil.profile === 'ribbon') {
		$('.cd-wirediameter').hide();
		$('.cd-wiredimensions').show();
		$('#cd-ribbonwidth').val(ds(coil.width,4,7));
		$('#cd-thickness').val(ds(coil.thickness,4,7));
		$('.cd-twistpitch').hide();
		$('.cd-strands').hide();
		$('cd-ribbonwidth').val(ds(coil.width,5,7));
		$('cd-thickness').val(ds(coil.thickness,5,7));
	} else {
		console.log('Unknown profile');
	}
	$('#cd-awg').val(coil.awg);
	if (coil.dia==='') {
		$('#cd-dia').val('').parents('.form-group').addClass('has-warning');
	} else {
		$('#cd-dia').val(ds(coil.dia)).parents('.form-group').removeClass('has-warning');
	}
	$('#cd-strands').val(coil.strands);
	$('#cd-setup').val(coil.setup);
	$('#cd-targetr').val(coil.targetr);
	$('#cd-innerdia').val(ds(coil.innerdia,2,3));
	$('#cd-leglength').val(ds(coil.leglength,1,3));
	$('#hfnw').val(coil.hfnw);
	$('#cd-wrapspace').val(ds(coil.wrapspace));
	$('#useneutralaxis').prop('checked',(coil.useNaDia===true));
	return;
}

function updateUnits() {
	if (coil.showSi) {
		$('.cd-im').hide();
		$('.cd-si').show();
		$('#cd-units-si').prop('checked',true);
	} else {
		$('.cd-si').hide();
		$('.cd-im').show();
		$('#cd-units-im').prop('checked',true);
	}
	return;
}

function toggleUi($toggel, $btn, setto, animate, timeoutvar) {
	clearTimeout(timeoutvar);
	if (setto) {
		if (animate) {
			$toggel.show('blind');
		} else {
			$toggel.show();
		}
		if ($btn) {
			$btn.addClass('active');
		}
	} else {
		if (animate) {
			$toggel.hide('blind');
		} else {
			$toggel.hide();
		}
		if ($btn) {
			$btn.removeClass('active');
		}
	}
	return;
}

function showMulticoilInfo() {
	$('.cd-setup-s').hide();
	$('.cd-setup-dp').hide();
	$('.cd-setup-ds').hide();
	$('.cd-setup-tp').hide();
	$('.cd-setup-ts').hide();
	$('.cd-setup-qp').hide();
	$('.cd-setup-qs').hide();
	$('.cd-setup-qsp').hide();
	if (coil.setup==='s') {
		$('.cd-setup-multi').hide();
		$('.cd-setup-single').show();
		$('#cd-len-number').html('').hide();
	} else {
		$('.cd-setup-single').hide();
		$('.cd-setup-multi').show();
		if (coil.setup==='dp') {
			$('.cd-setup-dp').show();
		} else if (coil.setup==='ds') {
			$('.cd-setup-ds').show();
		} else if (coil.setup==='tp') {
			$('.cd-setup-tp').show();
		} else if (coil.setup==='ts') {
			$('.cd-setup-ts').show();
		} else if (coil.setup==='qp') {
			$('.cd-setup-qp').show();
		} else if (coil.setup==='qs') {
			$('.cd-setup-qs').show();
		} else if (coil.setup==='qsp') {
			$('.cd-setup-qsp').show();
		}
		$('#cd-len-number').html(' (&times; ' +  $('#cd-setup').find('option:selected').attr('data-setup-wires') + ' wires)').show();
	}
	return;
}

function toggleBasicAdvanced(animate) {
	toggleUi($('.cd-advanced'), $('#cd-basicadvanced'), coil.advanced, animate, coil.showbasicanimtimeout);
	toggleUi($('.cd-adv'), false, coil.advanced, false);
	return;
}

function importCoil(imp) {
	//console.log(coil);
	var x;
	// aliaser
	for (x in imp) {
		if (imp.hasOwnProperty(x)) {
			if (x==='ll') { imp.leglength = imp.ll; }
			if (x==='id') { imp.innerdia = imp.id; }
			if (x==='r') { imp.targetr = imp.r; }
			if (x==='s') { imp.setup = imp.s; }
			if (x==='a') { imp.advanced = imp.a; }
			if (x==='si') { imp.showSi = imp.si; }
			if (x==='p') { imp.profile = imp.p; }
			if (x==='tp') { imp.twistpitch = imp.tp; }
			if (x==='str') { imp.strands = imp.str; }
			if (x==='wl') { imp.wirelength = imp.wl; }
			if (x==='ws') { imp.wrapspace = imp.ws; }
		}
	}
	// typekonvertering
	for (x in imp) {
		if (imp.hasOwnProperty(x)) {
			if (imp[x]==='') {
			} else if (imp[x]==='true') {
				coil[x] = true;
			} else if (imp[x]==='false') {
				coil[x] = false;
			} else if (!isNaN(imp[x])) {
				coil[x] = readNumber(imp[x]);
			} else {
				coil[x] = imp[x];
			}
		}
	}
	updateUnits();
	ccWrite();
	toggleBasicAdvanced(true);
	if (imp.awg) {
		coil.dia = awg2mm(imp.awg);
		writeDia();
	} else if (imp.dia) {
		coil.awg = '';
		$('#cd-awg').val('');
		writeDia();
	}
	findResistivity(coil);
	findTwist(coil);
	if (imp.rstv) {
		coil.mat = 'c';
		$('#cd-material').val(coil.mat);
		coil.rstv = imp.rstv;
		writeRstv(coil);
		/*
		if (coil.showSi) {
			$('#cd-rstv').val(coil.rstv);
		} else {
			$('#cd-rstv').val(in2mm(coil.rstv));
		}
		*/
	}
	if (imp.wirelength) {
		coil.wirelength = imp.wirelength;
		$('#cd-len').val(ds(coil.wirelength));
	} else {
		wireLength(coil);
	}
	wraps();
	return;
}




function mm2awg(mm) {
	function finn(fra, til) {
		var ii, awgmm, lt, hit, gt;
		for (ii=fra;ii<til;ii++) {
			awgmm = awg2mm(ii);
			if (awgmm > mm) {
				lt = ii;
			} else if (awgmm === mm) {
				hit = ii;
			} else if (awgmm < mm) {
				gt = ii;
				return {
					lt: lt
					,hit: hit
					,gt: gt
				};
			}
		}
		return;
	}
	var res = finn(-50, 500);
	if (res === undefined) {
		//console.log('leter videre');
		res = finn(-5000, 50000);
	}
	//console.log(res);
	return res;
}


function findClosestGauge(arr, searchv, searchii, getii) {
	var ii, res = {};
	if (getii === undefined) { getii = 0; }
	if (searchii === undefined) { searchii = 1; }
	for (ii=0;ii<arr.length;ii++) {
		if (arr[ii][searchii] > searchv) {
			res.lt = arr[ii][getii];
		} else if (arr[ii][searchii] === searchv) {
			res = {};
			res.hit = arr[ii][getii];
			return res;
		}
		if (arr[ii][searchii] < searchv) {
			res.gt = arr[ii][getii];
			return res;
		}
	}
	return res;
}
function findGauge(arr, searchv, searchii, getii) {
	var ii;
	if (searchii === undefined) { searchii = 0; }
	if (getii === undefined) { getii = 1; }
	for (ii=0;ii<arr.length;ii++) {
		if (String(arr[ii][searchii]) === String(searchv)) {
			return arr[ii][getii];
		}
	}
	return;
}

function writeGaugeResult(con, arr, nm) {
	if (nm === undefined) {
		nm = 'GA';
	}
	var cg, tmp1, tmp2;
	cg = findClosestGauge(arr, mm2in(con.mm));
	var x = '';
	if (cg.hit !== undefined) {
		x =  '<strong>' + cg.hit + ' '+nm+'</strong>';
	} else if (cg.lt !== undefined && cg.gt !== undefined) {
		tmp1 = findGauge(arr, cg.gt);
		tmp2 = findGauge(arr, cg.lt);
		var tmpPerc1 = (100-((con.mm/in2mm(tmp1))*100));
		var tmpPerc2 = (100-((con.mm/in2mm(tmp2))*100));
		x = '<i>between</i>';
		x += '<br />';
		x +=  '<strong>' + cg.gt + ' '+nm+'</strong> (';
		if (con.useUnit === 'mm') {
			x += round(in2mm(tmp1),con.tod);
		} else {
			x += round(tmp1,con.tod);
		}
		x += ' '+con.useUnit+')';
		x += ' <span class="label label-';
		if (Math.abs(tmpPerc1) < Math.abs(tmpPerc2)) {
			x += 'success';
		} else {
			x += 'default';
		}
		x += '">';
		if (tmpPerc1 > 0) { x += '+'; }
		x += round(tmpPerc1);
		x += '%</span>';
		x += '<br /><i>and</i></br>';
		x += '<strong>' + cg.lt + ' '+nm+'</strong> (';
		if (con.useUnit === 'mm') {
			x += round(in2mm(tmp2),con.tod);
		} else {
			x += round(tmp2,con.tod);
		}
		x += ' '+con.useUnit+')';
		x += ' <span class="label label-';
		if (Math.abs(tmpPerc2) < Math.abs(tmpPerc1)) {
			x += 'success';
		} else {
			x += 'default';
		}
		x += '">';
		if (tmpPerc2 > 0) { x += '+'; }
		x += round(tmpPerc2);
		x += '%</span>';
	} else {
		if (cg.lt !== undefined) {
			tmp1 = arr[arr.length-1][1];
		} else { // if (cg.gt !== undefined) {
			tmp1 = arr[0][1];
		}
		if (con.useUnit === 'mm') {
			tmp1 = in2mm(tmp1);
		}
		if (cg.lt !== undefined) {
			x = 'input too small, min ' + round(tmp1, con.tod) + ' ' + con.useUnit;
		} else { // if (cg.gt !== undefined) {
			x = 'input too large, max ' + round(tmp1, con.tod) + ' ' + con.useUnit;
		}
	}
	return x;
}

function xor(a, b) {
	if (a === true && b !== true) {
		return true;
	}
	if (a !== true && b === true) {
		return true;
	}
	return false;
}
function runConverter(con) {
	var txt = '';
	var txtv = '';
	var note = '';
	if (con.fromu === 'mm' || con.fromu === 'in') {
		con.useUnit = con.fromu;
	} else if (coil.showSi) {
		con.useUnit = 'mm';
	} else {
		con.useUnit = 'in';
	}
	// Konverter til mm
	if (con.fromu === 'mm') {
		con.mm = con.fromv;
	} else if (con.fromu === 'in') {
		con.mm = in2mm(con.fromv);
	} else if (con.fromu === 'awg') {
		con.mm = awg2mm(con.fromv);
	} else if (con.fromu === 'bwg') {
		con.mm = in2mm(findGauge(con.bwgArr, con.fromv));
	} else if (con.fromu === 'swg') {
		con.mm = in2mm(findGauge(con.swgArr, con.fromv));
	} else if (con.fromu === 'fracin') {
		con.mm = in2mm(findGauge(con.fracInArr, con.fromv));
	}
	// Konverter mm til endelig verdi
	if (con.tou === 'mm') {
		con.res = con.mm;
	} else if (con.tou === 'in') {
		con.res = mm2in(con.mm);
	} else if (con.tou === 'awg') {
		con.res = mm2awg(con.mm);
		//console.log(con.res);
		if (con.res === undefined) {
			txt = 'Cannot compute';
		} else if (con.res.hit !== undefined) {
			txt = 'AWG <strong>' + con.res.hit + '<strong>';
			if (con.res.hit < 1) {
				note = 'AWG below 1 is not returned in the standard format';
			}
		} else {
			txt = (function() {
				var x = '';
				// regn lt om til valgt ikke-awg-enhet
				// regn gt om til valgt ikke-awg-enhet
				x += '<strong>' + con.res.gt + '</strong> AWG';
				if (con.useUnit === 'in') {
					x += ' (' + round(awg2in(con.res.gt), con.tod) + ' in)';
				} else {
					x += ' (' + round(awg2mm(con.res.gt), con.tod) + ' mm)';
				}
				x += ' &mdash; ';
				x += '<strong>' + con.res.lt + '</strong> AWG';
				if (con.useUnit === 'in') {
					x += ' (' + round(awg2in(con.res.lt), con.tod) + ' in)';
				} else {
					x += ' (' + round(awg2mm(con.res.lt), con.tod) + ' mm)';
				}
				if (con.res.gt < 1) {
					note = 'Gauges larger than AWG 1 are returned as negative numbers instead of the multiple zeros of the standard format';
				}
				return x;
			}());
		}
	} else if (con.tou === 'bwg') {
		txt = writeGaugeResult(con, con.bwgArr, 'BWG');
	} else if (con.tou === 'swg') {
		txt = writeGaugeResult(con, con.swgArr, 'SWG');
	} else if (con.tou === 'fracin') {
		txt = writeGaugeResult(con, con.fracInArr, 'in');
	}
	// lag resultatstreng
	if (txt === '') {
		txt = '<strong>' + round(con.res, con.tod) + ' ' + con.tou + '</strong>';
	}
	// lag spørringstreng
	txtv = con.fromv + ' ';
	if (con.fromu === 'mm' || con.fromu === 'in') {
		txtv += con.fromu;
	} else if (con.fromu === 'fracin') {
		txtv += ' in';
	} else {
		txtv += con.fromu.toUpperCase();
	}
	if (coil.showSi && con.tou === 'mm') {
		// ikke vis mm i spørsmål, det kommer uansett i svar
	} else if (!coil.showSi && con.tou === 'in') {
		// ikke vis in i spørsmål, det kommer uansett i svar
	} else if (coil.showSi && con.fromu === 'in') {
		txtv += ' ('+round(in2mm(con.fromv), con.tod)+' mm)';
	} else if (!coil.showSi && con.fromu === 'mm') {
		txtv += ' ('+round(mm2in(con.fromv), con.tod)+' in)';
	} else if (con.fromu === 'awg') {
		if (con.useUnit === 'in') {
			txtv += ' ('+round(awg2in(con.fromv), con.tod)+' in)';
		} else {
			txtv += ' ('+round(awg2mm(con.fromv), con.tod)+' mm)';
		}
	} else if (con.fromu === 'bwg') {
		if (coil.showSi && con.tou !== 'mm') {
			txtv += ' ('+round(in2mm(parseFloat(findGauge(con.bwgArr, con.fromv))), con.tod)+' mm)';
		} else if (!coil.showSi && con.tou !== 'in') {
			txtv += ' ('+round(findGauge(con.bwgArr, con.fromv), con.tod)+' in)';
		}
	} else if (con.fromu === 'swg') {
		if (coil.showSi && con.tou !== 'mm') {
			txtv += ' ('+round(in2mm(parseFloat(findGauge(con.swgArr, con.fromv))), con.tod)+' mm)';
		} else if (!coil.showSi && con.tou !== 'in') {
			txtv += ' ('+round(findGauge(con.swgArr, con.fromv), con.tod)+' in)';
		}
	} else if (con.fromu === 'fracin') {
		if (coil.showSi && con.tou !== 'mm') {
			txtv += ' ('+round(in2mm(parseFloat(findGauge(con.fracInArr, con.fromv))), con.tod)+' mm)';
		} else if (!coil.showSi && con.tou !== 'in') {
			txtv += ' ('+round(findGauge(con.fracInArr, con.fromv), con.tod)+' in)';
		}
	}
	
	// skriv
	$('#conres').html('<small class="text-muted">' + txtv + ' =</small><br />' + txt);
	if (note === '') {
		$('#connote').hide();
	} else {
		$('#connote').html('Note: ' + note).show();
	}

	return;
}
function conFillSel(arr) {
	var ii, x = '';
	for (ii = 0; ii < arr.length; ii++) {
		x += '<option value="'+arr[ii][0]+'">'+arr[ii][0]+'</option>';
	}
	return x;
}
function initConverter() {
	var con = coil.converter;
	con.bwgArr = [ // format: [ bwg, in ]
		['00000 (5/0)', 0.500]
		,['0000 (4/0)', 0.454]
		,['000 (3/0)', 0.425]
		,['00 (2/0)', 0.380]
		,['0', 0.340]
		,['1', 0.300]
		,['2', 0.284]
		,['3', 0.259]
		,['4', 0.238]
		,['5', 0.220]
		,['6', 0.203]
		,['6.5', 0.188]
		,['7', 0.180]
		,['7.5', 0.172]
		,['8', 0.165]
		,['8.5',0.156]
		,['9', 0.148]
		,['9.5',0.141]
		,['10', 0.134]
		,['10.5',0.126]
		,['11', 0.120]
		,['11.5',0.115]
		,['12', 0.109]
		,['12.5',0.1]
		,['13', 0.095]
		,['13.5',0.089]
		,['14', 0.083]
		,['14.5',0.078]
		,['15', 0.072]
		,['15.5',0.068]
		,['16', 0.065]
		,['16.5',0.062]
		,['17', 0.058]
		,['17.5',0.056]
		,['18', 0.049]
		,['18.5',0.046]
		,['19', 0.042]
		,['19.5',0.039]
		,['20', 0.03575]
		,['20.5',0.343]
		,['21', 0.03225]
		,['21.5',0.0303]
		,['22', 0.02825]
		,['22.5',0.0263]
		,['23', 0.02525]
		,['23.5',0.0233]
		,['24', 0.02225]
		,['24.5',0.0213]
		,['25', 0.02025]
		,['25.5',0.0193]
		,['26', 0.01825]
		,['27', 0.01625]
		,['28', 0.01425]
		,['29', 0.01325]
		,['30', 0.01225]
		,['31', 0.01025]
		,['32', 0.00925]
		,['33', 0.00825]
		,['34', 0.00725]
		,['35', 0.005]
		,['36', 0.004]
	];

	con.fracArr = [
		'1/64' 
		,'1/32' 
		,'3/64' 	
		,'1/16' 
		,'5/64' 
		,'3/32' 
		,'7/64' 
		,'1/8' 
		,'9/64' 
		,'5/32' 
		,'11/64' 
		,'3/16' 
		,'13/64' 
		,'7/32' 
		,'15/64' 
		,'1/4' 
		,'17/64' 
		,'9/32' 
		,'19/64' 
		,'5/16' 
		,'21/64' 
		,'11/32' 
		,'23/64' 
		,'3/8' 
		,'25/64' 
		,'13/32' 
		,'27/64' 
		,'7/16' 
		,'29/64' 
		,'15/32' 
		,'31/64' 
		,'1/2' 
		,'33/64' 
		,'17/32' 
		,'35/64' 
		,'9/16' 
		,'37/64' 
		,'19/32' 
		,'39/64' 
		,'5/8' 
		,'41/64' 
		,'21/32' 
		,'43/64' 
		,'11/16' 
		,'45/64' 
		,'23/32' 
		,'47/64' 
		,'3/4' 
		,'49/64' 
		,'25/32' 
		,'51/64' 
		,'13/16' 
		,'53/64' 
		,'27/32' 
		,'55/64' 
		,'7/8' 
		,'57/64' 
		,'29/32' 
		,'59/64' 
		,'15/16' 
		,'61/64' 
		,'31/32' 
		,'63/64' 
	];
	con.fracInArr = []; // format: [ 'x/y', desimaltall ]
	(function() { // fyll ut fracInArr
		var ii, tmp;
		for (ii=0; ii < con.fracArr.length; ii++) {
			tmp = con.fracArr[ii].split('/');
			//con.fracInArr.push([ con.fracArr[ii], parseInt(tmp[0],10)/parseInt(tmp[1],10) ]);
			con.fracInArr.push([ '<sup>' + tmp[0] + '</sup>/<sub>' + tmp[1] + '</sub>', parseInt(tmp[0],10)/parseInt(tmp[1],10) ]);
		}
	}());
	con.fracInArr.reverse();
	//console.log(con.fracInArr);
	
	con.swgArr = [
		['7/0', 0.500]
		,['6/0', 0.464]
		,['5/0', 0.432]
		,['4/0', 0.400]
		,['3/0', 0.372]
		,['2/0', 0.348]
		,['0', 0.324]
		,['1', 0.300]
		,['2', 0.276]
		,['3', 0.252]
		,['4', 0.232]
		,['5', 0.212]
		,['6', 0.192]
		,['7', 0.176]
		,['8', 0.160]
		,['9', 0.144]
		,['10', 0.128]
		,['11', 0.116]
		,['12', 0.104]
		,['13', 0.092]
		,['14', 0.080]
		,['15', 0.072]
		,['16', 0.064]
		,['17', 0.056]
		,['18', 0.048]
		,['19', 0.040]
		,['20', 0.0369]
		,['21', 0.032]
		,['22', 0.028]
		,['23', 0.024]
		,['24', 0.022]
		,['25', 0.020]
		,['26', 0.018]
		,['27', 0.0164]
		,['28', 0.0148]
		,['29', 0.0136]
		,['30', 0.0124]
		,['31', 0.0116]
		,['32', 0.0108]
		,['33', 0.0100]
		,['34', 0.0092]
		,['35', 0.0084]
		,['36', 0.0076]
		,['37', 0.0068]
		,['38', 0.0060]
		,['39', 0.0052]
		,['40', 0.0048]
		,['41', 0.0044]
		,['42', 0.004]
		,['43', 0.0036]
		,['44', 0.0032]
		,['45', 0.0028]
		,['46', 0.0024]
		,['47', 0.0020]
		,['48', 0.0016]
		,['49', 0.0012]
		,['50', 0.0010]
	];
	
	// standardverdier
	con.fromv = 1;
	con.fromu = 'in';
	con.fromvStep = 0.001;
	con.tou = 'mm';
	con.tod = 3;
	// skriv og bind
	$('#confromu').val(con.fromu).change(function() {
		con.fromu = $(this).val();
		$('#confromvsel').hide();
		$('#confromv').show();
		con.fromv = $('#confromv').val();
		if (con.fromu === 'awg') {
			con.fromvStep = 1;
		} else if (con.fromu === 'in') {
			con.fromvStep = 0.001;
		} else if (con.fromu === 'bwg') {
			$('#confromv').hide();
			$('#confromvsel').show().html(conFillSel(con.bwgArr));
			con.fromv = $('#confromvsel').val();
		} else if (con.fromu === 'swg') {
			$('#confromv').hide();
			$('#confromvsel').show().html(conFillSel(con.swgArr));
			con.fromv = $('#confromvsel').val();
		} else if (con.fromu === 'fracin') {
			$('#confromv').hide();
			$('#confromvsel').show().html(conFillSel(con.fracInArr));
			con.fromv = $('#confromvsel').val();
		} else {
			con.fromvStep = 0.1;
		}
		$('#confromv').attr('step', con.fromvStep);
		runConverter(con);
		gad('coil-con', 'fromu', con.fromu);
	});
	$('#confromv').val(con.fromv).attr('step', con.fromvStep).change(function() {
		con.fromv = readNumber($(this).val());
		if (isNaN(con.fromv)) { con.fromv = 1; $(this).val(con.fromv); }
		runConverter(con);
		gad('coil-con', 'fromv', con.fromv);
	});
	$('#confromvsel').change(function() {
		con.fromv = $(this).val();
		runConverter(con);
		gad('coil-con','fromvsel', con.fromv);
	});
	$('#contou').val(con.tou).change(function() {
		con.tou = $(this).val();
		runConverter(con);
		gad('coil-con', 'tou', con.tou);
	});
	$('#contod').val(con.tod).change(function() {
		con.tod = parseInt($(this).val(),10);
		if (isNaN(con.tod)) { con.tod = 3; $(this).val(con.tod); }
		runConverter(con);
		gad('coil-con', 'tod', con.tod);
	});
	$('.cd-units').change(function() {
		runConverter(con);
	});
	// kjør
	runConverter(con);
	gad('coil-con','init');
	return;
}
function validerTwistPitch(coil) {
	var $fg = $('#cd-twistpitch').parents('.form-group');
	var $tpwarn = $fg.find('.tpwarn');
	if (coil.twistpitch <= coil.dia && coil.twistpitch !== 0) {
		$fg.addClass('has-error');
		if ($tpwarn.length === 0) {
			$fg.append('<span class="label label-danger tpwarn">The twist pitch must be greater than the wire diameter, or zero.</span>');
		}
	} else {
		$fg.removeClass('has-error').find('.tpwarn').remove();
	}
	return;
}

$(document).ready(function() {
	$('#cd-basicadvanced').click(function() {
		coil.advanced = !coil.advanced;
		toggleBasicAdvanced(true);
		if ($('#cd-int-export').is(':visible')) { exportSettings(); }
		gad('coil', 'advanced', coil.advanced);
	}).mouseenter(function() {
		$('.cd-advanced').addClass('bg-warning');
	}).mouseleave(function() {
		$('.cd-advanced').removeClass('bg-warning');
	});
	$('.cd-units').change(function() {
		coil.showSi = (!coil.showSi);
		updateUnits();
		ccWrite();
		writeDia();
		writeRstv(coil);
		findResistivity(coil);
		//console.log(coil.rstv);
		findTwist(coil);
		wireLength(coil);
		wraps();
		gad('coil', 'showsi', coil.showSi);
	});
	$('#cd-material').change(function() {
		coil.mat = $(this).val();
		if (coil.mat==='c') {
			coil.advanced = true;
			toggleBasicAdvanced(true);
		}
		writeDia();
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		gad('coil', 'mat', coil.mat);
	});
	$('#cd-profile').change(function() {
		coil.profile = $(this).val();
		if (coil.profile==='round') {
			//coil.ribbon = false;
			coil.strands = 1;
		} else if (coil.profile==='roundmulti') {
			//coil.ribbon = false;
			if (coil.strands < 2) { coil.strands = 2; }
		} else if (coil.profile==='ribbon') {
			//coil.ribbon = true;
			coil.strands = 1;
		} else {
			console.log('Unknown profile');
		}
		ccWrite();
		writeDia();
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		gad('coil', 'profile', $(this).val());
	});
	$('#cd-awg').change(function() {
		var awg = parseInt($(this).val(),10);
		if (awg<-10) {
			awg = -10;
			$(this).val(awg);
		} else if (awg>100) {
			awg = 100;
			$(this).val(awg);
		}
		$(this).parents('.form-group').removeClass('has-warning');
		coil.awg = awg;
		coil.dia = awg2mm(coil.awg);
		writeDia();
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		validerTwistPitch(coil);
		gad('coil', 'awg', coil.awg);
	});
	$('#cd-dia').change(function() {
		clearTimeout(coil.keyuptimeout);
		coil.dia = readNumber($(this).val());
		if (!coil.showSi) { coil.dia = in2mm(coil.dia); }
		$(this).parents('.form-group').removeClass('has-warning');
		coil.awg = '';
		$('#cd-awg').val('').parents('.form-group').addClass('has-warning');
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		validerTwistPitch(coil);
		gad('coil', 'dia', coil.dia);
	});
	$('#cd-strands').change(function() {
		coil.strands = parseInt($(this).val(),10);
		if (coil.twistpitch !== 0 && coil.strands > 4) {
			coil.strands = 4;
			$(this).val(4);
		}
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		gad('coil', 'pitch', coil.twistpitch);
		$('.cd-twistpitch-output').html(coil.strands);
	});
	$('#cd-twistpitch').change(function() {
		coil.twistpitch = readNumber($(this).val()); // antall mm på langs per rotasjon, per trådpar
		if (!coil.showSi) {
			coil.twistpitch = in2mm(coil.twistpitch);
		}
		if (coil.twistpitch !== 0 && coil.strands > 4) {
			coil.strands = 4;
			$('#cd-strands').val(4);
		}
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		validerTwistPitch(coil);
		gad('coil', 'pitch', coil.twistpitch);
	});
	function ribbonCheck() {
		var $div = $('.cd-wiredimensions');
		if (coil.width<coil.thickness) {
			$div.addClass('has-warning');
			$('#thicknesswarning').show();
		} else {
			$div.removeClass('has-warning');
			$('#thicknesswarning').hide();
		}
		return;
	}
	$('#cd-ribbonwidth').change(function() {
		coil.width = readNumber($(this).val());
		if (!coil.showSi) { coil.width = in2mm(coil.width); }
		ribbonCheck();
		findResistivity(coil);
		wireLength(coil);
		wraps();
		gad('coil', 'ribbonwidth', coil.width);
	});
	$('#cd-thickness').change(function() {
		coil.thickness = readNumber($(this).val());
		if (!coil.showSi) { coil.thickness = in2mm(coil.thickness); }
		ribbonCheck();
		findResistivity(coil);
		wireLength(coil);
		wraps();
		gad('coil', 'thickness', coil.thickness);
	});
	$('#thicknesswarning .btn').click(function() {
		var tmp = coil.thickness;
		coil.thickness = coil.width;
		coil.width = tmp;
		ccWrite();
		ribbonCheck();
		findResistivity(coil);
		wireLength(coil);
		wraps();
	});
	$('#cd-setup').change(function() {
		coil.setup = $(this).val();
		showMulticoilInfo();
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		gad('coil', 'setup', coil.setup);
	});
	$('#cd-targetr').change(function() {
		clearTimeout(coil.keyuptimeout);
		coil.targetr = readNumber($(this).val());
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		gad('coil', 'targetr', coil.targetr);
	});
	$('#cd-rstv').change(function() {
		clearTimeout(coil.keyuptimeout);
		coil.rstv = readNumber($(this).val());
		coil.mat = 'c';
		$('#cd-material').val(coil.mat);
		if (!coil.showSi) {
			coil.rstv = mm2in(coil.rstv);
		}
		wireLength(coil);
		wraps();
		gad('coil', 'rstv', coil.rstv);
	});
	$('#cd-len').change(function() {
		clearTimeout(coil.keyuptimeout);
		coil.wirelength = readNumber($(this).val());
		if (!coil.showSi) {
			coil.wirelength = in2mm(coil.wirelength);
		}
		wraps();
		gad('coil', 'wirelength', coil.wirelength);
	});
	$('#cd-leglength').change(function() {
		clearTimeout(coil.keyuptimeout);
		coil.leglength = readNumber($(this).val());
		if (!coil.showSi) {
			coil.leglength = in2mm(coil.leglength);
		}
		wraps();
		gad('coil', 'leglength', coil.leglength);
	});
	$('#cd-wrapspace').change(function() {
		clearTimeout(coil.keyuptimeout);
		coil.wrapspace = readNumber($(this).val());
		if (!coil.showSi) {
			coil.wrapspace = in2mm(coil.wrapspace);
		}
		wraps();
		gad('coil', 'wrapspace', coil.wrapspace);
	});
	$('#hfnw').change(function() {
		coil.hfnw = $(this).val();
		if (isNaN(coil.hfnw)) {
			coil.hfnw = 10;
			$(this).val(coil.hfnw);
		} else {
			coil.hfnw = readNumber(coil.hfnw);
		}
		wraps();
	});
	$('#cd-innerdia').change(function() {
		clearTimeout(coil.keyuptimeout);
		coil.innerdia = readNumber($(this).val());
		if (!coil.showSi) {
			coil.innerdia = in2mm(coil.innerdia);
		}
		$('#cd-innerdia-presets .btn').removeClass('active');
		wraps();
		gad('coil', 'innerdia', coil.innerdia);
	});
	$('#cc-resetdefaults').click(function() {
		coil = defaultCoil();
		updateUnits();
		ccWrite();
		writeDia();
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		toggleBasicAdvanced(true);
		ccMenuMsg('Default settings have been restored.','success');
		gad('coil', 'reset');
	});
	$('.fractioninputdiv').each(function() {
		setupFracInput($(this));
	});
	$('#useneutralaxis').click(function() {
		coil.useNaDia = ($(this).is(':checked'));
		wraps();
		exportSettings();
	});
	$('input, select').change(function() {
		exportSettings();
	});
	$('.btn').click(function() {
		exportSettings();
	});
	toggleBasicAdvanced(false);
	var qs = getQueryStrings();
	function endOfInit() {
		updateUnits();
		toggleBasicAdvanced(true);
		ccWrite();
		writeDia();
		findResistivity(coil);
		findTwist(coil);
		wireLength(coil);
		wraps();
		return;
	}
	choo.showSi = false;
	if (Object.keys(qs).length>0) {
		importCoil(qs);
		ccMenuMsg('Settings have been imported.','success');
		gad('coil', 'importqs', JSON.stringify(qs));
	} else {
		endOfInit();
	}
	showMulticoilInfo();
	// tabs
	$('#rescontab a').click(function (e) {
		e.preventDefault();
		$(this).tab('show');
	});
	$('#rescontab ul.nav li a[href="#converter"]').click(function() {
		if (coil.conInited !== true) {
			initConverter();
			coil.conInited = true;
		}
	});
});